from string import Template

ControllerTemplate = Template(
r"""import { Mutable } from '@savitri/backend'
import { ${pascal_case}Document, ${pascal_case} } from './${camel_case}.model'
import { default as Description } from './index.json'

/**
    @remarks Generated by savitrigen v${savitrigen_version}
    @copyright ${copyright}
    ${documentation}
*/

export class ${pascal_case}Controller extends Mutable<${pascal_case}Document> {
  constructor() {
    super(${pascal_case}, Description)
  }
}
""")

